@@ -1,5 +1,6 @@
 #!/usr/bin/env bash
-# lesspipe.sh, a preprocessor for less (version 2.04)
+# lesspipe.sh, a preprocessor for less
+lesspipe_version=2.08
 # Author: Wolfgang Friebel (wp.friebel AT gmail.com)
 #( [[ -n 1 && -n 2 ]] ) > /dev/null 2>&1 || exec zsh -y --ksh-arrays -- "$0" ${1+"$@"}
 
@@ -20,17 +21,19 @@ filetype () {
 	# do not depend on the file extension, if possible
 	fname="$1"
 	if [[ "$1" == - || -z $1 ]]; then
-		declare t=$(nexttmp)
-		head -c 40000 > "$t" 2>/dev/null
+		declare t
+		t=$(nexttmp)
+		head -c 1000000 > "$t" 2>/dev/null
 		set "$t" "$2"
 		fname="$fileext"
 	fi
 	fext=$(fileext "$fname")
 	### get file type from mime type
-	declare ft=$(file -L -s -b --mime "$1" 2> /dev/null)
+	declare ft
+	ft=$(file -L -s -b --mime "$1" 2> /dev/null)
 	[[ $ft == *=* ]] && fchar="${ft##*=}" || fchar=utf-8
 	fcat="${ft%/*}"
-	ft="${ft#*/}"; ft="${ft%;*}"; ft="${ft#x-}"
+	ft="${ft#*/}"; ft="${ft%;*}"; ft="${ft#x-script.}"; ft="${ft#x-}"
 	ftype="${ft#vnd\.}"
 	# chose better name
 	case "$ftype" in
@@ -40,7 +43,7 @@ filetype () {
 			ftype=pptx ;;
 		openxmlformats-officedocument.spreadsheetml.sheet)
 			ftype=xlsx ;;
-		oasis.opendocument.text)
+		oasis.opendocument.text*)
 			ftype=odt ;;
 		oasis.opendocument.spreadsheet)
 			ftype=ods ;;
@@ -49,16 +52,18 @@ filetype () {
 		sun.xml.writer)
 			ftype=ooffice1 ;;
 		shellscript)
-			ftype=sh ;;
+			ftype='sh' ;;
 		makefile)
-			ftype=make ;;
+			ftype='make' ;;
 		epub+zip)
 			ftype=epub ;;
+		matlab-data)
+			ftype=matlab ;;
 	# file may report wrong type for given file names (ok in file 5.39)
 		troff)
 			case "${fname##*/}" in
 			[Mm]akefile|[Mm]akefile.*|BSDMakefile)
-				ftype=make ;;
+				ftype='make' ;;
 			esac
 	esac
 	# correct for a more specific file type
@@ -71,6 +76,13 @@ filetype () {
 			[[ $ftype == mpeg ]] && ftype=mp3 ;;
 	esac
 	### get file type from 'file' command for an unspecific result
+	if [[ "$fcat" == message && $ftype == plain ]]; then
+		ftype=msg
+	fi
+	if [[ "$fcat" == message && $ftype == rfc822 ]]; then
+		fcat=text
+		ftype=email
+	fi
 	if [[ "$fcat" == application && "$ftype" == octet-stream || "$fcat" == text && $ftype == plain ]]; then
 		ft=$(file -L -s -b "$1" 2> /dev/null)
 		# first check if the file command yields something
@@ -79,18 +91,24 @@ filetype () {
 				ftype=matlab ;;
 			*POD\ document*)
 				ftype=pod ;;
-			*perl\ Storable*)
-				ftype=pst ;;
 			*PEM\ certificate\ request)
 				ftype=csr ;;
 			*PEM\ certificate)
 				ftype=csr ;;
+			*Microsoft\ OOXML)
+				ftype=docx ;;
 			Apple\ binary\ property\ list)
 				ftype=plist ;;
 			PGP\ *ncrypted*|GPG\ encrypted*)
 				ftype=pgp ;;
 			Audio\ file\ with\ ID3\ *)
 				ftype=mp3 ;;
+			OpenOffice.org\ 1.x\ Writer\ document)
+				ftype=ooffice1 ;;
+			*osascript*)
+				ftype=applescript ;;
+			*Device\ Tree\ Blob*)
+				ftype=dtb ;;
 			# if still unspecific, determine file type by extension
 			data)
 				### binary only file formats, type not guessed by 'file'
@@ -131,21 +149,29 @@ filetype () {
 				log)
 					ftype=log ;;
 				ebuild|eclass)
-					ftype=sh ;;
+					ftype='sh' ;;
 			esac
 		fi
 	fi
-	echo $ftype:$fchar:$fcat
+
+	echo "$ftype:$fchar:$fcat"
 }
 
 msg () {
 	[[ -n "$LESSQUIET" ]] && return;
-	echo "==> $@"
+	if [[ -n "$lesspipe_version" ]]; then
+		echo "==> (lesspipe $lesspipe_version) $*"
+	else
+		echo "==> $*"
+	fi
+	lesspipe_version=
 }
 
-contentline () {
+separatorline () {
 	declare a="==================================="
-	echo "$a Contents $a"
+	word="Contents"
+	[[ -n $1 ]] && word=$1
+	echo "$a $word $a"
 }
 
 nexttmp () {
@@ -179,14 +205,14 @@ show () {
 		isfinal "$1"
 		return
 	fi
-	file1="${1%%$sep*}"
+	file1="${1%%"$sep"*}"
 	rest1="${1#"$file1"}"
-	while [[ "$rest1" == $sep$sep* ]]; do
-		if [[ "$rest1" == $sep$sep ]]; then
+	while [[ "$rest1" == "$sep$sep"* ]]; do
+		if [[ "$rest1" == "$sep$sep" ]]; then
 			break
 		else
-			rest1="${rest1#$sep$sep}"
-			file1="${rest1%%$sep*}"
+			rest1="${rest1#"$sep$sep"}"
+			file1="${rest1%%"$sep"*}"
 			rest1="${rest1#"$file1"}"
 			file1="${1%"$rest1"}"
 		fi
@@ -194,15 +220,15 @@ show () {
 	if [[ ! -e "$file1" && "$file1" != '-' ]]; then
 		exit 1
 	fi
-	rest11="${rest1#$sep}"
-	file2="${rest11%%$sep*}"
+	rest11="${rest1#"$sep"}"
+	file2="${rest11%%"$sep"*}"
 	rest2="${rest11#"$file2"}"
-	while [[ "$rest2" == $sep$sep* ]]; do
-		if [[ "$rest2" == $sep$sep ]]; then
+	while [[ "$rest2" == "$sep$sep"* ]]; do
+		if [[ "$rest2" == "$sep$sep" ]]; then
 			break
 		else
-			rest2="${rest2#$sep$sep}"
-			file2="${rest2%%$sep*}"
+			rest2="${rest2#"$sep$sep"}"
+			file2="${rest2%%"$sep"*}"
 			rest2="${rest2#"$file2"}"
 			file2="${rest11%"$rest2"}"
 		fi
@@ -210,21 +236,20 @@ show () {
 	rest2="${rest11#"$file2"}"
 	rest11="$rest1"
 
-	if [[ "$cmd" == "" ]]; then
+	if [[ "${cmd[*]}" == "" ]]; then
 		ft=$(filetype "$file1")
-		get_unpack_cmd $ft "$file1" "$rest1"
-		if [[ "$cmd" != "" && -z $colorizer ]]; then
+		get_unpack_cmd "$ft" "$file1" "$rest1"
+		if [[ "${cmd[*]}" != "" ]]; then
 			show "-$rest1"
 		else
 			# if nothing to convert, exit without a command
-			[[ $colorizer == cat ]] && colorizer=
 			isfinal "$file1" "$rest11"
 		fi
 	elif [[ "$c1" == "" ]]; then
 		c1=("${cmd[@]}")
 		ft=$("${c1[@]}" | filetype -) || exit 1
-		get_unpack_cmd $ft "$file1" "$rest1"
-		if [[ "$cmd" != "" && -z $colorizer ]]; then
+		get_unpack_cmd "$ft" "$file1" "$rest1"
+		if [[ "${cmd[*]}" != "" ]]; then
 			show "-$rest1"
 		else
 			"${c1[@]}" | isfinal - "$rest11"
@@ -232,8 +257,8 @@ show () {
 	elif [[ "$c2" == "" ]]; then
 		c2=("${cmd[@]}")
 		ft=$("${c1[@]}" | "${c2[@]}" | filetype -) || exit 1
-		get_unpack_cmd $ft "$file1" "$rest1"
-		if [[ "$cmd" != "" && -z $colorizer ]]; then
+		get_unpack_cmd "$ft" "$file1" "$rest1"
+		if [[ "${cmd[*]}" != "" ]]; then
 			show "-$rest1"
 		else
 			"${c1[@]}" | "${c2[@]}" | isfinal - "$rest11"
@@ -241,8 +266,8 @@ show () {
 	elif [[ "$c3" == "" ]]; then
 		c3=("${cmd[@]}")
 		ft=$("${c1[@]}" | "${c2[@]}" | "${c3[@]}" | filetype -) || exit 1
-		get_unpack_cmd $ft "$file1" "$rest1"
-		if [[ "$cmd" != "" && -z $colorizer ]]; then
+		get_unpack_cmd "$ft" "$file1" "$rest1"
+		if [[ "${cmd[*]}" != "" ]]; then
 			show "-$rest1"
 		else
 			"${c1[@]}" | "${c2[@]}" | "${c3[@]}" | isfinal - "$rest11"
@@ -250,8 +275,8 @@ show () {
 	elif [[ "$c4" == "" ]]; then
 		c4=("${cmd[@]}")
 		ft=$("${c1[@]}" | "${c2[@]}" | "${c3[@]}" | "${c4[@]}" | filetype -) || exit 1
-		get_unpack_cmd $ft "$file1" "$rest1"
-		if [[ "$cmd" != "" && -z $colorizer ]]; then
+		get_unpack_cmd "$ft" "$file1" "$rest1"
+		if [[ "${cmd[*]}" != "" ]]; then
 			show "-$rest1"
 		else
 			"${c1[@]}" | "${c2[@]}" | "${c3[@]}" | "${c4[@]}" | isfinal - "$rest11"
@@ -259,8 +284,8 @@ show () {
 	elif [[ "$c5" == "" ]]; then
 		c5=("${cmd[@]}")
 		ft=$("${c1[@]}" | "${c2[@]}" | "${c3[@]}" | "${c4[@]}" | "${c5[@]}" | filetype -) || exit 1
-		get_unpack_cmd $ft "$file1" "$rest1"
-		if [[ "$cmd" != "" && -z $colorizer ]]; then
+		get_unpack_cmd "$ft" "$file1" "$rest1"
+		if [[ "${cmd[*]}" != "" ]]; then
 			echo "$0: Too many levels of encapsulation"
 		else
 			"${c1[@]}" | "${c2[@]}" | "${c3[@]}" | "${c4[@]}" | "${c5[@]}" | isfinal - "$rest11"
@@ -272,7 +297,7 @@ get_unpack_cmd () {
 	fchar="${1%:*}"; fchar="${fchar#*:}"
 	fcat="${1##*:}"
 	x="${1%%:*}"
-	cmd=
+	cmd=()
 	if [[ "$3" == $sep$sep ]]; then
 		return
 	fi
@@ -284,7 +309,7 @@ get_unpack_cmd () {
 			[[ $2 == - ]] || fileext="$2"
 			fileext=${fileext%%.gz}; fileext=${fileext%%.bz2}
 			[[ $x == compress ]] && x=gzip
-			has_cmd $x && cmd=($x -cd "$2") && return ;;
+			has_cmd "$x" && cmd=("$x" -cd "$2") && return ;;
 		zstd)
 			has_cmd zstd && cmd=(zstd -cdqM1073741824 "$2") && return ;;
 		lz4)
@@ -298,21 +323,21 @@ get_unpack_cmd () {
 	esac
 	# convert into utf8
 
-	if [[ -n $lclocale && $fchar != binary && $fchar != *ascii && $fchar != $lclocale && $fchar != unknown* ]]; then
+	if [[ -n $lclocale && $fchar != binary && $fchar != *ascii && $fchar != "$lclocale" && $fchar != unknown* ]]; then
 		qm="\033[7m?\033[m" # inverted question mark
-		rep=-c
-		trans=
-		echo ""|iconv --byte-subst - 2>/dev/null && rep="--unicode-subst=$qm --byte-subst=$qm --widechar-subst=$qm" # MacOS
-		echo ""|iconv -f $fchar -t $locale//TRANSLIT - 2>/dev/null && trans="-t $locale//TRANSLIT"
-		msg "append $sep$sep to filename to view the $fchar encoded file"
-		cmd=(iconv $rep -f $fchar $trans "$2")
+		rep=(-c)
+		trans=()
+		echo ""|iconv --byte-subst - 2>/dev/null && rep=(--unicode-subst="$qm" --byte-subst="$qm" --widechar-subst="$qm") # MacOS
+		echo ""|iconv -f "$fchar" -t "$locale//TRANSLIT" - 2>/dev/null && trans=(-t "$locale//TRANSLIT")
+		msg "append $sep$sep to filename to view the original $fchar encoded file"
+		cmd=(iconv "${rep[@]}" -f "$fchar" "${trans[@]}" "$2")
 		# loop protection, just in case
 		lclocale=
 		return
 	fi
-	[[ "$3" == $sep ]] && return
-	file2=${3#$sep}
-	file2=${file2%%$sep*}
+	[[ "$3" == "$sep" ]] && return
+	file2=${3#"$sep"}
+	file2=${file2%%"$sep"*}
 	# remember name of file to extract or file type
 	[[ -n "$file2" ]] && fileext="$file2"
 	# extract from archive
@@ -346,20 +371,20 @@ get_unpack_cmd () {
 			{ has_cmd bsdtar && prog=bsdtar; } ||
 			{ has_cmd isoinfo && prog=isoinfo; } ;;
 		archive)
-			prog=ar
+			prog='ar'
 			has_cmd bsdtar && prog=bsdtar
 	esac
-	[[ -n $prog ]] && cmd=(isarchive $prog "$2" "$file2")
-	if [[ -n $cmd ]]; then
+	[[ -n $prog ]] && cmd=(isarchive "$prog" "$2" "$file2")
+	if [[ -n ${cmd[*]} ]]; then
 		[[ -n "$file2" ]] && file2= && return
 		msg "use ${x}_file${sep}contained_file to view a file in the archive"
-		has_cmd archive_color && colorizer=archive_color || colorizer=cat
+		has_cmd archive_color && colorizer=(archive_color)
 	fi
 }
 
 analyze_args () {
 	# determine how we are called
-	cmdtree=`ps -T -oargs=`
+	cmdtree=$(ps -T -oargs= 2>/dev/null)
 	while read -r line; do
 		arg1=${line%% *}; arg1=${arg1##*/}
 		case $arg1 in
@@ -373,50 +398,65 @@ analyze_args () {
 	# return if we want to watch growing files
 	[[ $lessarg == *less\ *\ +F\ * || $lessarg == *less\ *\ : ]] && exit 0
 	# color is set when calling less with -r or -R or LESS contains that option
-	lessarg="l $LESS $lessarg"
-	lessarg=`echo $lessarg|sed 's/ -[a-zA-Z]*[rR]/ -r/'`
+	COLOR="--color=auto"
 	has_cmd tput && colors=$(tput colors) || colors=0
-	if [[ $colors -ge 8 && $lessarg == *\ -[rR]* ]]; then
-		COLOR="--color=always"
-	else
-		COLOR="--color=auto"
+	if [[ $colors -ge 8 ]]; then
+		# shellcheck disable=SC2206
+		r_string=($LESS $lessarg)
+		for i in "${r_string[@]}"
+		do
+			[[ $i = -*[rR] ]] && COLOR="--color=always"
+			[[ $i = --raw-control-chars ]] && COLOR="--color=always"
+		done
 	fi
 	# last argument starting with colon or equal sign is used for piping into less
 	[[ $lessarg == *\ [:=]* ]] && fext=${lessarg#*[:=]}
 }
 
 has_colorizer () {
-	arg="$1"
-	[[ "$arg" = - ]] && arg=
 	[[ $COLOR == *always ]] || return
 	[[ $2 == plain || -z $2 ]] && return
 	prog=${LESSCOLORIZER%% *}
 
 	for i in bat batcat pygmentize source-highlight code2color vimcolor ; do
-		[[ -z $prog || $prog == $i ]] && has_cmd $i && prog=$i
+		[[ -z $prog || $prog == "$i" ]] && has_cmd "$i" && prog=$i
 	done
-	[[ "$2" =~ ^[0-9]*$ ]] && opt= || opt=" -l $2"
+	[[ "$2" =~ ^[0-9]*$ || -z "$2" ]] || lang=$2
+	# prefer an explicitly requested language
+	[[ -n $3 ]] && lang=$3 || lang=$2
 	case $prog in
 		bat|batcat)
-			# only allow an explicitly requested language
-			opt=" -l $fileext"
-			{ [[ -n $fileext ]] && $prog $opt /dev/null; } || opt=
-			opt="$opt $COLOR" ;;
+			[[ -n $lang ]] && $prog --list-languages|grep -i "$lang" > /dev/null && opt=(-l "$lang")
+			[[ -n $LESSCOLORIZER && $LESSCOLORIZER = *\ *--style=* ]] && style="${LESSCOLORIZER/* --style=/}"
+			[[ -z $style ]] && style=$BAT_STYLE
+			[[ -z $style ]] && style=plain
+			[[ -n $LESSCOLORIZER && $LESSCOLORIZER = *\ *--theme=* ]] && theme="${LESSCOLORIZER/* --theme=/}"
+			[[ -z $theme ]] && theme=$BAT_THEME
+			[[ -z $theme ]] && theme=ansi
+			if [[ -r "$HOME/.config/bat/config" ]]; then
+				grep -q -e '^--style' "$HOME/.config/bat/config" || opt+=(--style="${style%% *}")
+				grep -q -e '^--theme' "$HOME/.config/bat/config" || opt+=(--theme="${theme%% *}")
+			fi
+			opt+=("$COLOR" --paging=never "$1") ;;
 		pygmentize)
-				[[ -n $LESSCOLORIZER && $LESSCOLORIZER =~ pygmentize\ \ *-O\ *style=[a-z]* ]] && prog=$LESSCOLORIZER
-				[[ $colors -ge 256 ]] && prog="$prog -f terminal256"
-				res=$(pygmentize -l $2 /dev/null 2>/dev/null) && opt=" -l $2" || opt=" -g" ;;
+			pygmentize -l "$lang" /dev/null &>/dev/null && opt=(-l "$lang") || opt=(-g)
+			[[ -n $LESSCOLORIZER && $LESSCOLORIZER = *-O\ *style=* ]] && style="${LESSCOLORIZER/*style=/}"
+			[[ -n $style ]] && opt+=(-O style="${style%% *}")
+			[[ $colors -ge 256 ]] && opt+=(-f terminal256)
+			[[ "$1" == - ]] || opt+=("$1") ;;
 		source-highlight)
-			prog="source-highlight --failsafe -f esc"
-			[[ -z $arg ]] && arg=/dev/stdin
-			[[ -n "$opt" && -n "$2" ]] && opt=" -s $2 -i" || opt=" -i" ;;
+			[[ -n $1 && "$1" != - ]] && opt=(-i "$1") || opt=()
+			[[ -n $lang ]] && opt+=(-s "$lang")
+			style=esc
+			[[ $colors -ge 256 ]] && style=esc256
+			opt+=(--failsafe -f "$style") ;;
 		code2color|vimcolor)
-			opt=
-			[[ -n "$fileext" ]] && opt=" -l $fileext" ;;
+			opt=("$1")
+			[[ -n "$3" ]] && opt=(-l "$3" "$1") ;;
 		*)
 			return ;;
 	esac
-	echo "$prog$opt $arg"
+	colorizer=("$prog" "${opt[@]}")
 }
 
 isfinal () {
@@ -424,21 +464,23 @@ isfinal () {
 		cat "$1"
 		return
 	fi
-	if [[ -z "$cmd" ]]; then
+	if [[ -z "${cmd[*]}" ]]; then
 	# respect extension set by user
 	[[ -n "$file2" && "$fileext" == "$file2" && "$fileext" != *.* ]] && x="$fileext"
 	case "$x" in
 		directory)
-			cmd=(ls -lA $COLOR "$1")
-			if ! ls $COLOR > /dev/null 2>&1; then
+			cmd=(ls -lA "$COLOR" "$1")
+			if ! ls "$COLOR" > /dev/null 2>&1; then
 				cmd=(CLICOLOR_FORCE=1 ls -lA -G "$1")
 				if ! ls -lA -G > /dev/null 2>&1; then
 					cmd=(ls -lA "$1")
 				fi
 			fi
-			msg="$x: showing the output of ${cmd[@]}" ;;
+			msg="$x: showing the output of ${cmd[*]}" ;;
 		html|xml)
 			[[ -z $file2 ]] && has_htmlprog && cmd=(ishtml "$1") ;;
+		dtb|dts)
+			has_cmd dtc && cmd=(isdtb "$1") ;;
 		pdf)
 			{ has_cmd pdftotext && cmd=(istemp pdftotext -layout -nopgbrk -q -- "$1" -); } ||
 			{ has_cmd pdftohtml && has_htmlprog && cmd=(istemp ispdf "$1"); } ||
@@ -447,10 +489,10 @@ isfinal () {
 			has_cmd ps2ascii && nodash ps2ascii "$1" ;;
 		java-applet)
 			# filename needs to end in .class
-			has_cmd procyon && t=$t.class && cat "$1" > $t && cmd=(procyon "$t") ;;
+			has_cmd procyon && t=$t.class && cat "$1" > "$t" && cmd=(procyon "$t") ;;
 		markdown)
-			[[ $COLOR = *always ]] && mdopt= || mdopt=-c
-			{ has_cmd mdcat && cmd=(mdcat $mdopt "$1"); } ||
+			[[ $COLOR = *always ]] && mdopt=() || mdopt=(-c)
+			{ has_cmd mdcat && cmd=(mdcat "${mdopt[@]}" "$1"); } ||
 			{ has_cmd pandoc && cmd=(pandoc -t plain "$1"); } ;;
 		docx)
 			{ has_cmd pandoc && cmd=(pandoc -f docx -t plain "$1"); } ||
@@ -467,13 +509,13 @@ isfinal () {
 			{ has_cmd xlscat && cmd=(istemp xlscat "$1"); } ||
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" xlsx); } ;;
 		odt)
-			{ has_cmd pandoc && cmd=(pandoc -f odt -t plain "$1"); } ||
 			{ has_cmd odt2txt && cmd=(istemp odt2txt "$1"); } ||
+			{ has_cmd pandoc && cmd=(pandoc -f odt -t plain "$1"); } ||
 			{ has_cmd libreoffice && cmd=(isoffice2 "$1"); } ;;
 		odp)
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" odp); } ;;
 		ods)
-			{ has_cmd xlscat && t=$t.ods && cat "$1" > $t && cmd=(xlscat "$t"); } ||
+			{ has_cmd xlscat && t=$t.ods && cat "$1" > "$t" && cmd=(xlscat "$t"); } ||
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" ods); } ;;
 		msword)
 			t="$1"; [[ "$t" == - ]] && t=/dev/stdin
@@ -490,14 +532,16 @@ isfinal () {
 			{ has_cmd sxw2txt && cmd=(istemp sxw2txt "$1"); } ||
 			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" odt); } ;;
 		ipynb|epub)
-			has_cmd pandoc && cmd=(pandoc -f $x -t plain "$1") ;;
+			has_cmd pandoc && cmd=(pandoc -f "$x" -t plain "$1") ;;
 		troff)
 			if has_cmd groff; then
 				fext=$(fileext "$1")
 				declare macro=andoc
-				[[ fext == me ]] && macro=e
-				[[ fext == ms ]] && macro=s
-				cmd=(groff -s -p -t -e -Tutf8 -m$macro "$1")
+				[[ "$fext" == me ]] && macro=e
+				[[ "$fext" == ms ]] && macro=s
+				cmd=(groff -s -p -t -e -Tutf8 -m "$macro" "$1")
+			elif has_cmd mandoc; then
+				cmd=(mandoc "$1")
 			fi ;;
 		rtf)
 			{ has_cmd unrtf && cmd=(istemp "unrtf --text" "$1"); } ||
@@ -507,12 +551,10 @@ isfinal () {
 		sharedlib)
 			cmd=(istemp nm "$1");;
 		pod)
-			[[ -z $file2 ]] && LESSQUIET=1 &&
+			[[ -z $file2 ]] &&
 			{ { has_cmd pod2text && cmd=(pod2text "$1"); } ||
 			{ has_cmd perldoc && cmd=(istemp perldoc "$1"); }; } ;;
-		pst)
-			has_cmd perl && perl -MStorable=retrieve -MData::Dumper -e '$Data::Dumper::Indent=1;print Dumper retrieve shift' "$1" ;;
-		hdf)
+		hdf|hdf5)
 			{ has_cmd h5dump && cmd=(istemp h5dump "$1"); } ||
 			{ has_cmd ncdump && cmd=(istemp ncdump "$1"); } ;;
 		matlab)
@@ -526,58 +568,69 @@ isfinal () {
 		pgp)
 			has_cmd gpg && cmd=(gpg -d "$1") ;;
 		plist)
-			has_cmd plistutil && cmd=(istemp "plistutil -i" "$1") ;;
+			{ has_cmd plistutil && cmd=(istemp "plistutil -i" "$1"); } ||
+			{ has_cmd plutil && cmd=(istemp "plutil -p" "$1"); } ;;
 		mp3)
 			has_cmd id3v2 && cmd=(istemp "id3v2 --list" "$1") ;;
 		log)
-			has_cmd ccze && [[ $COLOR = *always ]] && cat "$1" | ccze -A
+			has_cmd ccze && [[ $COLOR = *always ]] && ccze -A < "$1"
 			return ;;
 		csv)
 			{ has_cmd csvlook && cmd=(csvlook "$1"); } ||
 			{ has_cmd pandoc && cmd=(pandoc -f csv -t plain "$1"); } ;;
+		json)
+			[[ $COLOR = *always ]] && opt=(-C .) || opt=(.)
+			has_cmd jq && cmd=(jq "${opt[@]}" "$1") ;;
 	esac
 	fi
 	# not a specific file format
-	if [[ -z "$cmd" ]]; then
+	if [[ -z ${cmd[*]} ]]; then
 		fext=$(fileext "$1")
 		if [[ $fcat == audio || $fcat == video || $fcat == image ]]; then
-			{ has_cmd mediainfo && cmd=(mediainfo --Full "$1"); } ||
+			{ [[ "$1" != '-' ]] && has_cmd mediainfo && cmd=(mediainfo --Full "$1"); } ||
 			{ has_cmd exiftool && cmd=(exiftool "$1"); } ||
-			{ has_cmd identify && $fcat == image && cmd=(identify -verbose "$1"); }
+			{ has_cmd identify && [[ $fcat == image ]] && cmd=(identify -verbose "$1"); }
 		elif [[ "$fchar" == binary ]]; then
 			cmd=(nodash strings "$1")
 		fi
 	fi
-	if [[ -n $cmd && $cmd != "cat" ]]; then
-		[[ -z $msg ]] && msg="append $sep to filename to view the $x file"
-		msg $msg
+	if [[ -n ${cmd[*]} && "${cmd[*]}" != "cat" ]]; then
+		[[ -z $msg ]] && msg="append $sep to filename to view the original $x file"
+		msg "$msg"
 	fi
-	if [[ -n $cmd ]]; then
-		if [[ $colorizer == archive_color && $COLOR == *always ]]; then
-			"${cmd[@]}" | archive_color
-		else
-			"${cmd[@]}"
+	[[ -n "$file2" ]] && fext="$file2"
+	[[ $fcat == text && $x != plain ]] && fext=$x
+	[[ -z "$fext" ]] && fext=$(fileext "$fileext")
+	fext=${fext##*/}
+	[[ -z ${colorizer[*]} ]] && has_colorizer "$1" "$fext" "$fileext"
+	if [[ -n ${cmd[*]} ]]; then
+		# TAU: When cmd starts with environment variable settings, bash will refuse to execute it via : "${cmd[@]}"
+		# The remedy is simple : Just run it through the "env" command in that case.
+		if [[ "$cmd" =~ '=' ]]; then
+			cmd=(env "${cmd[@]}")
 		fi
+		#[[ -n ${colorizer[*]} ]] && "${cmd[@]}" | "${colorizer[@]}" && return
+		"${cmd[@]}"
 	else
-		[[ -n "$file2" ]] && fext="$file2"
-		[[ -z "$fext" && $fcat == text && $x != plain ]] && fext=$x
-		[[ -z "$fext" ]] && fext=$(fileext "$fileext")
-		fext=${fext##*/}
-		[[ -z $colorizer ]] && colorizer=$(has_colorizer "$1" "$fext")
-		[[ -n $colorizer && $fcat != binary ]] && $colorizer && return
+		[[ -n ${colorizer[*]} && $fcat != binary ]] && "${colorizer[@]}" && return
 		# if fileext set, we need to filter to get rid of .fileext
-		[[ -n $fileext || "$1" == - || "$1" == $t ]] && cat "$1"
+		[[ -n $fileext || "$1" == - || "$1" == "$t" ]] && cat "$1"
 	fi
 }
 
 isarchive () {
 	prog=$1
-	[[ "$2" =~ ^[a-z_-]*:.* ]] && echo $2: remote operation tar host:file not allowed && return
+	[[ "$2" =~ ^[a-z_-]*:.* ]] && echo "$2: remote operation tar host:file not allowed" && return
 	if [[ -n $3 ]]; then
 		case $prog in
 			tar|bsdtar)
-				[[ "$2" =~ ^[a-z_-]*:.* ]] && echo $2: remote operation tar host:file not allowed && return
-				$prog Oxf "$2" "$3" 2>/dev/null;;
+				[[ "$2" =~ ^[a-z_-]*:.* ]] && echo "$2: remote operation tar host:file not allowed" && return
+				if [[ "$3" =~ --* ]]; then
+					$prog Oxf "$2" "--" "$3" 2>/dev/null
+				else
+					$prog Oxf "$2" "$3" 2>/dev/null
+				fi
+				;;
 			rar|unrar)
 				istemp "$prog p -inul" "$2" "$3" ;;
 			ar)
@@ -594,7 +647,7 @@ isarchive () {
 	else
 		case $prog in
 			tar|bsdtar)
-				[[ "$2" =~ ^[a-z_-]*:.* ]] && echo $2: remote operation tar host:file not allowed && return
+				[[ "$2" =~ ^[a-z_-]*:.* ]] && echo "$2: remote operation tar host:file not allowed" && return
 				$prog tvf "$2" ;;
 			rar|unrar)
 				istemp "$prog v" "$2" ;;
@@ -608,12 +661,13 @@ isarchive () {
 				t="$2"
 				istemp "isoinfo -d -i" "$2"
 				isoinfo -d -i "$t"| grep -E '^Joliet' && joliet=J
-				contentline
-				isoinfo -fR$joliet -i "$t" ;;
+				separatorline
+				isoinfo -fR"$joliet" -i "$t" ;;
 			7za|7zr)
 				istemp "$prog l" "$2"
 		esac
 	fi
+	[[ $? != 0 && -n $3 ]] && msg ":$!: could not retrieve $3 from $2"
 }
 
 cabextract2 () {
@@ -628,7 +682,7 @@ isrpm () {
 	if [[ -z "$2" ]]; then
 		if has_cmd rpm; then
 			istemp "rpm -qivp" "$1"
-			contentline
+			separatorline
 			[[ $1 == - ]] && set "$t" "$1"
 		fi
 		if has_cmd bsdtar; then
@@ -653,8 +707,8 @@ isdeb () {
 		data=$(bsdtar tf "$1" "data*")
 		if [[ -z "$2" ]]; then
 			control=$(bsdtar tf "$1" "control*")
-			bsdtar xOf "$1" $control | bsdtar xOf - ./control
-			contentline
+			bsdtar xOf "$1" "$control" | bsdtar xOf - ./control
+			separatorline
 			bsdtar xOf "$1" "$data" | bsdtar tvf -
 		else
 			bsdtar xOf "$1" "$data" | bsdtar xOf - "$2"
@@ -662,14 +716,14 @@ isdeb () {
 	else
 		data=$(ar t "$1"|grep data)
 		ft=$(ar p "$1" "$data" | filetype -)
-		get_unpack_cmd $ft -
+		get_unpack_cmd "$ft" -
 		if [[ -z "$2" ]]; then
 			control=$(ar t "$1"|grep control)
-			ar p "$1" $control | ${cmd[@]} | tar xOf - ./control
-			contentline
-			ar p "$1" "$data" | ${cmd[@]} | tar tvf -
+			ar p "$1" "$control" | "${cmd[@]}" | tar xOf - ./control
+			separatorline
+			ar p "$1" "$data" | "${cmd[@]}" | tar tvf -
 		else
-			ar p "$1" "$data" | ${cmd[@]} | tar xOf - "$2"
+			ar p "$1" "$data" | "${cmd[@]}" | tar xOf - "$2"
 		fi
 	fi
 }
@@ -677,15 +731,26 @@ isdeb () {
 isoffice () {
 	t=$(nexttmp)
 	t2=$t."$2"
-	cat "$1" > $t2
+	cat "$1" > "$t2"
 	libreoffice --headless --convert-to html --outdir "$tmpdir" "$t2" > /dev/null 2>&1
-	ishtml $t.html
+	ishtml "$t.html"
 }
 
 isoffice2 () {
 	istemp "libreoffice --headless --cat" "$1" 2>/dev/null
 }
 
+isdtb () {
+	errors=$(nexttmp)
+	has_colorizer "-" "dts" "dts"
+	[[ -z "${colorizer[*]}" ]] && colorizer=(cat)
+	dtc -I dtb -O dts -o - -- "$1" 2> "$errors" | "${colorizer[@]}"
+	if [[ -s "$errors" ]]; then
+		separatorline "Warnings"
+		cat "$errors"
+	fi
+}
+
 has_htmlprog () {
 	if has_cmd w3m || has_cmd lynx || has_cmd elinks || has_cmd html2text; then
 		return 0
@@ -696,7 +761,7 @@ has_htmlprog () {
 handle_w3m () {
 	if [[ "$1" == *\?* ]]; then
 		t=$(nexttmp)
-		ln -s "$1" $t
+		ln -s "$1" "$t"
 		set "$t" "$1"
 	fi
 	nodash "w3m -dump -T text/html" "$1"
@@ -720,30 +785,30 @@ setopt sh_word_split 2>/dev/null
 PATH=$PATH:${0%%/lesspipe.sh}
 # the current locale in lowercase (or generic utf-8)
 locale=$(locale|grep LC_CTYPE|sed 's/.*"\(.*\)"/\1/') || locale=en_US.UTF-8
-lclocale=$(echo ${locale##*.}|tr '[A-Z]' '[a-z]')
+lclocale=$(echo "${locale##*.}"|tr '[:upper:]' '[:lower:]')
 
 sep=:					# file name separator
-altsep==				# alternate separator character
-if [[ -e "$1" && "$1" == *$sep* ]]; then
+altsep='='				# alternate separator character
+if [[ -e "$1" && "$1" == *"$sep"* ]]; then
 	sep=$altsep
-elif [[ "$1" == *$altsep* ]]; then
-	[[ -e "${1%%$altsep*}" ]] && sep=$altsep
+elif [[ "$1" == *"$altsep"* ]]; then
+	[[ -e "${1%%"$altsep"*}" ]] && sep=$altsep
 fi
 
 tmpdir=${TMPDIR:-/tmp}/lesspipe."$RANDOM"
 [[ -d "$tmpdir" ]] || mkdir "$tmpdir"
 [[ -d "$tmpdir" ]] || exit 1
-trap "rm -rf '$tmpdir';exit 1" INT
-trap "rm -rf '$tmpdir'" EXIT
+trap 'rm -rf "$tmpdir";exit 1' INT
+trap 'rm -rf "$tmpdir"' EXIT
 trap - PIPE
 
 t=$(nexttmp)
 analyze_args
 # make LESSOPEN="|- ... " work
 if [[ $LESSOPEN == *\|-* && $1 == - ]]; then
-	cat > $t
+	cat > "$t"
 	[[ -n "$fext" ]] && t="$t$sep$fext"
-	set $1 "$t"
+	set "$1" "$t"
 	nexttmp >/dev/null
 fi
 
